from gameprices.offer import GameOffer


class Shop(object):
    """A generic video game online store"""

    def __init__(self, country):
        super(Shop, self).__init__()
        self.country = country

    def search(self, name):
        super(Shop, self).search()



import datetime
import logging
import sys
from urllib.parse import quote

from gameprices.offer import GameOffer, Price
from gameprices.shop import Shop
from gameprices.utils import utils

api_root = "https://store.playstation.com/chihiro-api"
store_root = "https://store.playstation.com/#!"
fetch_size = "99999"
appendix = ""
api_version = "19"

version = sys.version_info[0]

""" Dictionary object containing data specific to a Country store. The key is the store identifier per the
PSN API. The array of parameters contains [0]: the country-code folder for the PSN Store URL and
[1]: The currency symbol, in its unicode encoding
[2]: The character preceding each marketplaces CID """
store_code_mappings = {
    "NL/nl": ["nl-nl", u"\N{EURO SIGN}"],
    "DE/de": ["de-de", u"\N{EURO SIGN}", "E"],
    "US/en": ["us-en", u"\N{DOLLAR SIGN}", "U"],
    "JP/jp": ["jp-jp", u"\N{YEN SIGN}", "J"],
}


def _filter_none(item):
    if item is None:
        return False
    else:
        return True


def _get_item_for_cid(cid, store):
    try:
        url = (api_root + "/viewfinder/" + store + "/" + api_version + "/" + cid + "?size=" + fetch_size)
        data = utils.get_json_response(url)
        return data
    except Exception as e:
        logging.error("Got error '" + str(e) + "' while retrieving cid '" + cid + "' in store " + store)
        return None


def _get_rewards(item):
    rewards = []

    for sku in item["skus"]:
        if "rewards" in sku:
            for reward in sku["rewards"]:
                rewards.append(reward)

    return rewards


def _get_display_price(item, store):
    price = _get_price(item)
    display_price = store_code_mappings[store][1] + str(price)

    return display_price


def _get_cheapest_price(prices):
    return sorted(filter(_filter_none, prices))[0]


def _get_price(item):
    normal_price = _get_normal_price(item)
    non_playstation_plus_price = _get_non_playstation_plus_price(item)
    playstation_plus_price = _get_playstation_plus_price(item)

    return _get_cheapest_price(
        [normal_price, non_playstation_plus_price, playstation_plus_price]
    )


def _get_normal_price(item):
    return float(item["default_sku"]["price"]) / 100 if "default_sku" in item else None


def _get_non_playstation_plus_price(item):
    for reward in _get_rewards(item):
        if reward.get("reward_type") == 4:
            return float(reward.get("price")) / 100


def _get_playstation_plus_price(item):
    for reward in _get_rewards(item):
        if reward.get("bonus_price") is not None:
            return float(reward.get("bonus_price")) / 100
        elif reward.get("isPlus") is True:
            return float(reward.get("price")) / 100
        else:
            return None

    return None


def _get_name(item):
    return item["name"]


def _get_image(item):
    if len(item["images"]) > 0:
        return item["images"][0]["url"]


def _get_offer_end_date(item):
    """ Returns the Offer End Date for a given Item
        :param item: The item for which the Offer End Date is to be retrieved
        :return: A datetime object which is the Offer End Date """

    if item["default_sku"] is not None:
        if "end_date" in item["default_sku"]:
            end_date = item["default_sku"]["end_date"]
            if end_date is not None:
                return datetime.datetime.strptime(end_date, "%Y-%m-%dT%H:%M:%SZ")


def _get_store_url(item, store):
    cid = item["id"]

    url = store_root + "/" + store_code_mappings[store][0] + "/cid=" + cid

    return url


def _get_cid_for_name(name, store):
    links = _search_for_items_by_name(name, store)
    cids = []

    for link in links:
        try:
            logging.debug("Parsing:\n" + utils.pretty_print_json(link))
            name = link["name"]
            item_type = link["default_sku"]["name"]
            cid = link["id"]
            platform = ", ".join(link["playable_platform"])

            logging.info("Found: " + name + " - " + cid + " - Platform: " + platform + " - Type: " + item_type)
            cids.append(cid)
        except Exception as e:
            logging.warning("Got error '" + str(e) + "'' while parsing\n" + utils.pretty_print_json(link))

    return cids


def _search_for_items_by_name(name, store):
    encoded_name = quote(name)
    url = api_root + "/bucket-search/" + store + "/" + api_version + "/" + encoded_name + "?size=" + fetch_size + "&start=0"
    data = utils.get_json_response(url)
    links = data["categories"]["games"]["links"]
    return links


def _get_currency_symbol(store):
    if store in store_code_mappings:
        return store_code_mappings.get(store)[1]
    else:
        return ""


def _determine_store(cid):
    for store in store_code_mappings:
        store_code_mapping = store_code_mappings.get(store)

        if len(store_code_mapping) >= 3 and cid.startswith(
                store_code_mappings.get(store)[2]
        ):
            return store


def _get_items_by_container(container, store, filters_dict):
    url = api_root + "/viewfinder/" + store + "/" + api_version + "/" + container + "?size=" + fetch_size

    for i in filters_dict:
        url = url + "&" + quote(i) + "=" + quote(filters_dict[i])

    data = utils.get_json_response(url)
    links = data["links"]

    return links


class Psn(Shop):
    @staticmethod
    def _build_api_url(country, query):
        return "%s/%s/select?q=%s&%s" % (api_root, country, query, appendix)

    def _item_to_game_offer(self, game):
        if not game:
            raise Exception("Item is empty")

        return GameOffer(
            id=game["id"],
            cid=game["id"],
            url=game["url"],
            type=game["gameContentTypesList"][0]["key"]
            if "gameContentTypesList" in game
            else None,
            name=game["name"],
            # prices=[game['default_sku']['price']/100],
            prices=[
                Price(
                    value=_get_normal_price(game),
                    currency=_get_currency_symbol(self.country),
                    offer_type="NORMAL",
                ),
                Price(
                    value=_get_playstation_plus_price(game),
                    currency=_get_currency_symbol(self.country),
                    offer_type="PS+",
                ),
            ],
            platforms=game["playable_platform"] if "playable_platform" in game else "",
            picture_url=_get_image(game),
        )

    def search(self, name):
        items = _search_for_items_by_name(name=name, store=self.country)
        return_offers = []
        for item in items:
            return_offers.append(self._item_to_game_offer(item))
        return return_offers

    def get_item_by(self, item_id):
        item = _get_item_for_cid(item_id, self.country)
        return self._item_to_game_offer(item)



import json

from gameprices.cli.cli import eshop_main
from gameprices.cli.cli import psn_main
from gameprices.shops.eshop import Eshop
from gameprices.shops.psn import Psn
import sys
import pytest
import unittest


def test_cli_search():
    sys.argv = ["eshopcli", "--query", "'Vostok'"]

    with pytest.raises(SystemExit) as pytest_wrapped_e:
        eshop_main()
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 0


def test_cli_by_id_and_whish_price_not_matched():
    sys.argv = [
        "eshopcli",
        "--id",
        "DE/de###1173281###Mario_Kart_8_Deluxe",
        "--price",
        "1",
    ]

    with pytest.raises(SystemExit) as pytest_wrapped_e:
        eshop_main()
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == -1


def test_cli_by_id_and_whish_price_matched():
    sys.argv = [
        "eshopcli",
        "--id",
        "DE/de###1173281###Mario_Kart_8_Deluxe",
        "--price",
        "100",
    ]

    with pytest.raises(SystemExit) as pytest_wrapped_e:
        eshop_main()
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 0


@unittest.skip("Not yet implemented")
def test_cli_by_container():
    sys.argv = ["psncli", "--container", "STORE-MSF75508-PRICEDROPSCHI"]

    with pytest.raises(SystemExit) as pytest_wrapped_e:
        psn_main()
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 0


def test_cli_json(capsys):
    sys.argv = ["psncli", "--json", "--query", "Metal Gear"]

    with pytest.raises(SystemExit) as pytest_wrapped_e:
        psn_main()

    captured = capsys.readouterr()
    response = json.loads(captured.out)
    assert response is not None

    # Check if response json is populated
    assert len(response[0]["name"]) > 0
    assert len(response[0]["prices"]) > 0
    assert len(response[0]["type"]) > 0

    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 0


def test_cli_normal(capsys):
    sys.argv = ["psncli", "--query", "Metal Gear"]

    with pytest.raises(SystemExit) as pytest_wrapped_e:
        psn_main()

    captured = capsys.readouterr()
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 0


def test_cli_no_match():
    sys.argv = [
        "psncli",
        "--query",
        "sdfjsdkfsdkfjskdfj YOU WONT FIND ME NEVER EVER. HOPEFULLY",
    ]

    with pytest.raises(SystemExit) as pytest_wrapped_e:
        psn_main()
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == -1



class GameOffer(object):

    id = None
    name = None
    prices = []
    platforms = []
    picture_url = None

    def __init__(
        self, id, cid, name, url, prices, platforms, picture_url=None, type=None
    ):
        super(GameOffer, self).__init__()
        self.prices = prices
        self.name = name
        self.id = id
        self.cid = cid
        self.url = url
        self.type = type
        # TODO
        self.picture_base_url = "TODO"
        self.picture_url = picture_url
        self.platforms = platforms

    def __str__(self):
        return "%s - %s on %s" % (self.name, self.prices, ",".join(self.platforms))

    def dump(self):
        return {
            "id": self.id,
            "name": self.name,
            "type": self.type,
            "prices": [o.dump() for o in self.prices],  # TODO Don't print null values
            "platforms": self.platforms,
        }

    def get_item_by(self, id, name):
        raise NotImplementedError

    def search(self, name):
        raise NotImplementedError

    def get_full_image(self):
        return self.picture_base_url + self.picture_url


class Price(object):
    def __init__(self, value, currency, offer_type):
        super(Price, self).__init__()
        self.value = value
        self.currency = currency
        self.offer_type = offer_type

    def dump(self):
        return {
            "price": self.value,
            "currency": self.currency,
            "type": self.offer_type,
        }



from _pytest.monkeypatch import MonkeyPatch
import pytest
import sys

monkeypatch = MonkeyPatch()


def mailalert(alerts_line, mailfunc, should_remain_in_file=""):

    import smtplib

    filename = "alerts.csv"

    f = open(filename, "w")
    f.write(alerts_line)
    f.close()

    f = open("mailconfig.json", "w")
    f.write(
        """
    {
        "from":"from@example.com",
        "to":"to@example.com",
        "username":"from_user",
        "password":"from_password",
        "server":"localhost"
    }
    """
    )
    f.close()

    def mocklogin(cl, user, password):
        return True

    def mockehlo(cl):
        pass

    def mockstarttls(cl):
        pass

    def mockinit(object, config):
        pass

    def mockquit(cl):
        pass

    def mockreturn(cl, frm, to, msg):
        assert "from@example.com" == frm
        assert "to@example.com" == to
        assert "Price Drop" in msg
        # TODO Does this really work?

    monkeypatch.setattr(smtplib.SMTP, "__init__", mockinit)
    monkeypatch.setattr(smtplib.SMTP, "login", mocklogin)
    monkeypatch.setattr(smtplib.SMTP, "starttls", mockstarttls)
    monkeypatch.setattr(smtplib.SMTP, "ehlo", mockehlo)
    monkeypatch.setattr(smtplib.SMTP, "sendmail", mockreturn)
    monkeypatch.setattr(smtplib.SMTP, "quit", mockquit)

    sys.argv = [
        "psnmailalert",
    ]
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        # Call parameter
        mailfunc()
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 0

    data = ""
    with open(filename, "r") as myfile:
        data = data + myfile.read()

    print("Remaining content of file '%s'" % data)

    assert should_remain_in_file == data



#!/usr/bin/env python

import argparse
import json
import logging
from typing import List

from gameprices.offer import GameOffer
from gameprices.shop import Shop
from gameprices.shops.eshop import Eshop
from gameprices.shops.psn import Psn
from gameprices.utils import utils

logging.basicConfig(
    filename="psnprices.log",
    level=logging.INFO,
    format="%(asctime)s [%(levelname)-8s] %(message)s",
    filemode="w",
)

parser = argparse.ArgumentParser()
parser.add_argument("--id", "-i", help="CID of game to check")
parser.add_argument(
    "--store",
    "-s",
    help="Regional PSN store to check. Default: 'DE/de'",
    default="DE/de",
)
parser.add_argument("--price", "-p", help="Desired price of game", type=float)
parser.add_argument("--query", "-q", help="Name of item to search for")
parser.add_argument(
    "--json", "-j", dest="json", action="store_true", default=False, help="Print JSON"
)
parser.add_argument(
    "--log", "-l", help="Write to log file", dest="log", action="store_true"
)


def check_wish_price(shop: Shop, cid: str, wish_price: float) -> bool:
    item = shop.get_item_by(item_id=cid)
    normal_price = item.prices[0].value
    name = item.name

    if normal_price > wish_price:
        utils.print_enc(("Wish price {0:.2f} for '" + name + "' does not yet match {1:.2f}, exiting").format(wish_price,
                                                                                                             normal_price))
        return False
    else:
        utils.print_enc(
            ("Wish price {0:.2f} for '" + name + "' matched. Is now: {1:.2f}").format(
                wish_price, normal_price
            )
        )
        return True


def format_items_as_text(items):
    cids = []
    found_items = []

    for item in items:
        try:
            name = item.name
            item_type = item.type
            cid = item.cid
            price = str(item.prices[0].value) if len(item.prices) > 0 else ""

            platform = ",".join(item.platforms)
            found_items.append(
                (cid + "\t" + name + "\t" + platform + "\t" + price + "\t" + item_type)
            )
            cids.append(cid)
        except Exception as e:
            logging.exception(e)

    return found_items


def format_items_as_json(items: List[GameOffer]) -> str:
    return json.dumps([o.dump() for o in items])


def search_for_items_by_name_and_format_output(shop: Shop, name: str, print_json: bool):
    items = shop.search(name)
    if print_json:
        return format_items_as_json(items)
    else:
        return format_items_as_text(items)


def main(shop):
    args = parser.parse_args()

    if not args.log:
        logging.getLogger().disabled = True

    if args.query is not None and args.store is not None:
        print_string = search_for_items_by_name_and_format_output(
            shop, args.query, args.json
        )
        if len(print_string) == 0:
            exit(-1)
        elif not args.json:
            utils.print_enc("\n".join(print_string))
            exit(0)
        elif args.json:
            print(print_string)
            exit(0)

    elif args.store is not None and args.id is not None and args.price is not None:
        price_matched = check_wish_price(shop, args.id, args.price)
        if price_matched:
            exit(0)
        else:
            exit(-1)


def eshop_main():
    main(Eshop("DE/de"))


def psn_main():
    main(Psn("DE/de"))



from gameprices.cli.mailalert import main as psnmailalert_main
from gameprices.test.commons import mailalert


def test_mailfunc_not_existing():
    wrong_line = "EP9000-CUSA07123_00-000000000000000,10.00,DE/de"
    mailalert(wrong_line, psnmailalert_main, should_remain_in_file=wrong_line)


def test_mailfunc_existing_and_not_existing():
    unmatchable_price = "EP9000-CUSA07123_00-NIOHEU0000000000,0.00,DE/de"
    matchable_and_unmatchable_price = (
        unmatchable_price + "\nEP9000-CUSA07123_00-NIOHEU0000000000,100.00,DE/de"
    )
    mailalert(
        matchable_and_unmatchable_price,
        psnmailalert_main,
        should_remain_in_file=unmatchable_price + "\n",
    )


def test_support_lines_without_store():
    mailalert("EP0177-CUSA07010_00-SONICMANIA000000,100.00", psnmailalert_main)



#!/usr/bin/env python

import csv
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from gameprices.shops import psn
from gameprices.shops.eshop import Eshop
from gameprices.shops.psn import Psn
from gameprices.utils import utils


def get_mail_config():
    mail_config = utils.get_json_file("mailconfig.json")
    return mail_config


def get_alerts(alerts_filename):
    alerts = []
    with open(alerts_filename) as csv_file:
        alerts_reader = csv.reader(csv_file, delimiter=",", quotechar='"')
        for row in alerts_reader:
            alert = {"cid": row[0], "price": row[1]}
            if len(row) >= 3:
                alert["store"] = row[2]
            # TODO wild hack and duplication of code
            elif "###" not in alert["cid"]:
                alert["store"] = psn._determine_store(alert["cid"])
            alerts.append(alert)

    return alerts


def set_alerts(filename, alerts):
    c = csv.writer(open(filename, "w"))
    for alert in alerts:
        c.writerow([alert["cid"], alert["price"], alert["store"]])


def alert_is_matched(alert, item):
    return item and float(item.prices[0].value) <= float(alert["price"])


def check_alerts_and_generate_mail_body(alerts):

    body_elements = []
    unmatched_alerts = list(alerts)

    for alert in alerts:
        cid = alert["cid"]
        store = alert["store"]

        if "###" in cid:
            shop = Eshop(store)
        else:
            shop = Psn(store)

        try:
            item = shop.get_item_by(item_id=cid)
        except Exception as e:
            print(
                "Did not find an item for id %s in store %s with exception '%s'"
                % (cid, store, e)
            )
            continue

        if alert_is_matched(alert, item):
            body_elements.append(generate_body_element(alert, item))

            unmatched_alerts.remove(alert)

    body = "\n".join(body_elements)

    return unmatched_alerts, body


def send_mail(body):

    mail_config = get_mail_config()

    msg = MIMEMultipart("alternative")
    msg["From"] = mail_config["from"]
    msg["To"] = mail_config["to"]
    msg["Subject"] = "PlayStation Network Price Drop"

    send_body = body

    html_mail = MIMEText(send_body, "html")
    msg.attach(html_mail)

    mail_server = smtplib.SMTP(mail_config["server"])
    mail_server.ehlo()
    mail_server.starttls()
    mail_server.ehlo()
    mail_server.login(mail_config["username"], mail_config["password"])
    mail_server.sendmail(mail_config["from"], msg["To"], msg.as_string())

    mail_server.quit()


def generate_body_element(alert, item):

    return_body = [
        "<p><img src='" + item.get_full_image() + "'/></p>",
        "<p>" + item.name + "</p>",
        "<p>Wished: " + str(alert["price"]) + "</p>",
        "<p>Is now: " + str(item.prices[0].value) + "</p>"
    ]

    return "\n".join(return_body)


def main():
    alerts_filename = "alerts.csv"
    alerts = get_alerts(alerts_filename)

    alerts_remaining, body = check_alerts_and_generate_mail_body(alerts)
    utils.print_enc("Finished processing")

    if len(body) > 0:
        send_mail(body)
        utils.print_enc("Mail was sent")
        set_alerts(alerts_filename, alerts_remaining)
    else:
        utils.print_enc("No mail was sent")

    exit(0)



import urllib.request
import urllib.parse

from gameprices.shop import Shop
from gameprices.offer import GameOffer, Price
from gameprices.utils import utils

apiRoot = ""
storeRoot = ""
fetchSize = ""
apiVersion = ""
country = "de"
appendix = ""
limit = 30


class Eshop(Shop):
    whitespace_replacer = "_"
    id_spacer = "###"

    @staticmethod
    def _build_api_url(api_country, query):
        return "%s/%s/select?q=%s&%s" % (apiRoot, api_country, query, appendix)

    def _encode_id(self, encode_id, name):
        return self.country + self.id_spacer + str(encode_id) + self.id_spacer + name.replace(" ", self.whitespace_replacer)

    def _decode_id(self, encoded_id):
        split_id = encoded_id.split(self.id_spacer)
        country_from_id = split_id[0]
        id_from_id = split_id[1]
        name_from_id = split_id[2].replace(self.whitespace_replacer, " ")
        return country_from_id, id_from_id, name_from_id

    def search(self, name):
        url = "https://search.nintendo-europe.com/" + country + "/select?q=" + urllib.parse.quote(name) + "&fq=type%3A*%20AND%20*%3A*&start=0&rows=24&wt=json&group=true&group.field=pg_s&group.limit=" + str(limit) + "&group.sort=score%20desc,%20date_from%20desc&sort=score%20desc,%20date_from%20desc"

        payload = utils.get_json_response(url)

        return_offers = []

        for group in payload["grouped"]["pg_s"]["groups"]:
            if group["groupValue"] == "GAME":
                for game in group["doclist"]["docs"]:
                    price = game["price_lowest_f"]
                    return_offers.append(
                        GameOffer(
                            id=game["fs_id"],
                            cid=self._encode_id(encode_id=game["fs_id"], name=game["title"]),
                            url=game["url"],
                            name=game["title"],
                            type=game["type"],
                            prices=[
                                Price(
                                    value=price if price > 0 else None,
                                    # TODO add currency
                                    currency="",
                                    offer_type="OFFER",
                                ),
                            ],
                            platforms=game["system_names_txt"],
                            picture_url=game["image_url"]
                            if "image_url" in game
                            else None,
                        )
                    )

        return return_offers

    def get_item_by(self, item_id):
        decoded_country, item_id, name = self._decode_id(item_id)
        games = self.search(name=name)
        for game in games:
            if game.id == item_id:
                return game



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from gameprices.shops import psn
from gameprices.shops.psn import Psn

from gameprices.cli.mailalert import main as psnmailalert_main
from gameprices.test.commons import mailalert


class PsnTest(unittest.TestCase):

    # CID for item that is free for Plus members but not for normal members
    # TuneIt on german store
    freeForPlusCid = "EP4423-PCSB00407_00-TUNEIN0000000000"
    psn = Psn(country="DE/de")

    def test_search_for_cid_by_title_in_german_store(self):
        cids = psn._get_cid_for_name("Metal Gear", "DE/de")

        assert len(cids) > 0

    def test_search_for_cid_by_title_in_us_store(self):
        cids = psn._get_cid_for_name("Metal Gear", "US/en")

        assert len(cids) > 0

    def test_get_item_for_cid(self):
        store = "DE/de"
        cids = psn._get_cid_for_name("Tearaway", store)
        item = psn._get_item_for_cid(cids[0], store)

        assert item["name"] is not None

    def test_get_item_for_cid2(self):
        store = "DE/de"
        cids = psn._get_cid_for_name("Child of Light", store)
        item = psn._get_item_for_cid(cids[0], store)

        assert item["name"] is not None

    def test_get_item_by_container(self):
        store = "DE/de"
        items = psn._get_items_by_container(
            "STORE-MSF75508-PLUSINSTANTGAME", store, {"platform": "ps4"}
        )

        assert len(items) > 0

    def test_get_playstation_plus_price(self):
        store = "DE/de"
        item = psn._get_item_for_cid(self.freeForPlusCid, store)

        print(
            "Using '"
            + item["name"]
            + "' ("
            + self.freeForPlusCid
            + ") from "
            + store
            + " for comparison. Item must be free for Plus members in order to pass the unit test. This might fail due to price changes"
        )

        assert item is not None

        normalPrice = psn._get_price(item)
        plusPrice = psn._get_playstation_plus_price(item)

        print(
            "Normal Price: ", "%.2f" % normalPrice, "Plus Price: ", "%.2f" % plusPrice
        )

        assert isinstance(normalPrice, float)
        assert isinstance(plusPrice, float)
        assert plusPrice == 0

    def test_check_currency_symbolAsPartOfName(self):
        store = "DE/de"
        cids = psn._get_cid_for_name("Child of Light", store)
        item = psn._get_item_for_cid(cids[0], store)
        assert psn._get_display_price(item, store)[0] == u"\N{EURO SIGN}"

    def test_check_currency_symbol(self):
        assert psn._get_currency_symbol("DE/de") == u"\N{EURO SIGN}"
        assert psn._get_currency_symbol("US/en") == u"\N{DOLLAR SIGN}"
        assert psn._get_currency_symbol("JP/jp") == u"\N{YEN SIGN}"
        assert psn._get_currency_symbol("Unknown") == ""

    def test_get_rewards(self):
        store = "DE/de"
        item = psn._get_item_for_cid("EP0006-CUSA02532_00-UNRAVELUNRAVEL09", store)
        assert len(psn._get_rewards(item)) > -1

    @unittest.skip("Skip temporary price reduction")
    def test_check_currently_reduced_item_all_prices(self):
        store = "DE/de"
        item = psn._get_item_for_cid("EP2107-CUSA00327_00-DONTSTARVEPS4V01", store)
        print("Checking: ", item["name"])
        assert psn._get_normal_price(item) == 13.99
        assert psn._get_playstation_plus_price(item) == 4.89
        assert psn._get_non_playstation_plus_price(item) == 6.99
        assert psn._get_price(item) == 4.89

    @unittest.skip("Skip temporary price reduction")
    def test_check_currently_reduced_item_no_plus_reduction(self):
        store = "DE/de"
        item = psn._get_item_for_cid("EP9000-CUSA00194_00-UNTILDAWN0000001", store)
        print("Checking: ", item["name"])
        assert psn._get_normal_price(item) == 59.99
        assert psn._get_playstation_plus_price(item) is None
        assert psn._get_non_playstation_plus_price(item) == 44.99
        assert psn._get_price(item) == 44.99

    def test_determine_storeFromCID(self):
        assert psn._determine_store("EP9000-CUSA00194_00-UNTILDAWN0000001") == "DE/de"
        assert psn._determine_store("JP0006-NPJB00377_00-BATTLEFIELD40000") == "JP/jp"
        assert psn._determine_store("UP2034-CUSA04841_00-NMSDIGITAL000001") == "US/en"
        assert psn._determine_store("1") is None

    def get_game(self, name="Bloodborne", id="EP9000-CUSA00207_00-BLOODBORNE0000EU"):
        game = self.psn.get_item_by(item_id=id)
        return game

    def test_get_item_for_id(self):
        game_offers = self.psn.search("Tearaway™ Unfolded")
        game_offer = game_offers[0]
        assert game_offer.name == "Tearaway™ Unfolded"

    def test_search_alot(self):
        game_offers = self.psn.search("park")
        print("\n".join(str(e) for e in game_offers))
        assert len(game_offers) > 1

    def test_search_test_without_playable_platforms(self):
        game_offers = self.psn.search("test")
        print("\n".join(str(e) for e in game_offers))
        assert len(game_offers) > 1

    def test_get_item_by_id(self):
        id = "EP9000-CUSA00562_00-TEARAWAYUNFOLDED"
        name = "Tearaway™ Unfolded"
        game_offer = self.psn.get_item_by(item_id=id)

        assert game_offer.name == name
        assert game_offer.id == id

    def test_game_has_picture(self):
        assert "http" in self.get_game().picture_url

    def test_mailfunc(self):
        mailalert("EP0177-CUSA07010_00-SONICMANIA000000,100.00", psnmailalert_main)



import unittest

from gameprices.test.commons import mailalert
from gameprices.cli.mailalert import main as psnmailalert_main
from gameprices.shops.eshop import Eshop


class EshopTest(unittest.TestCase):

    eshop = Eshop(country="DE")

    def test_get_item_for_id(self):
        game_offers = self.eshop.search("Celeste")
        game_offer = game_offers[0]
        assert game_offer.name == "Celeste"

    def test_search_alot(self):
        game_offers = self.eshop.search("a")
        print("\n".join(str(e) for e in game_offers))
        assert len(game_offers) > 1

    def test_id_encoder(self):
        assert "DE###1207064###Celeste_123" == self.eshop._encode_id(
            encode_id=1207064, name="Celeste 123"
        )

    def test_id_decoder(self):
        assert ("DE", "1207064", "Celeste 123") == self.eshop._decode_id(
            "DE###1207064###Celeste_123"
        )

    def test_mailfunc(self):
        mailalert("DE/de###1174779###Sonic_Mania,100.00,DE/de", psnmailalert_main)



from setuptools import setup

try:
    import pypandoc

    long_description = pypandoc.convert("README.md", "rst")
except (IOError, ImportError):
    long_description = open("README.md").read()

tests_require = [
    "pytest",
]

setup(
    name="gameprices",
    packages=[
        "gameprices",
        "gameprices.shops",
        "gameprices.cli",
        "gameprices.test",
        "gameprices.utils",
    ],
    version="1.2",
    description="An interface for the undocumented Sony PlayStation Store PSN and Nintendo Eshop Apis",
    author="Matthias Kuech",
    author_email="post@matthias-kuech.de",
    url="https://github.com/snipem/psnprices",
    download_url="https://github.com/snipem/psnprices/archive/1.0.tar.gz",
    keywords=["playstation", "eshop", "store", "prices"],
    license="GPL2",
    test_suite="gameprices.test",
    tests_require=tests_require,
    entry_points={
        "console_scripts": [
            "eshopcli=gameprices.cli.cli:eshop_main",
            "psncli=gameprices.cli.cli:psn_main",
            "psnmailalert=gameprices.cli.mailalert:main",
            "dealsmailalert=gameprices.cli.mailalert:main",
        ]
    },
)



import json
import urllib.request


def get_json_file(filename):
    json_data = open(filename)
    data = json.load(json_data)
    json_data.close()
    return data


def get_json_response(url):

    with urllib.request.urlopen(url) as response:
        return json.loads(response.read())


def pretty_print_json(json_string):
    return json.dumps(json_string, sort_keys=True, indent=4, separators=(",", ": "))


def print_enc(s):
    print(s.decode("utf-8") if isinstance(s, type(b"")) else s)
